@@ -30,7 +30,11 @@
 
 const QString BLD_DATE(__DATE__);
 const QString BLD_TIME(__TIME__);
+#if defined(Q_CC_GNU)
 const QString CMP_VERSION(__VERSION__);
+#elif defined(Q_CC_MSVC)
+const QString CMP_VERSION(QString::number(_MSC_VER/100.0,'g',2));
+#endif
 
 const QString QSTR_APPNAME("Virtual MIDI Piano Keyboard");
 const QString QSTR_DOMAIN("vmpk.sourceforge.net");
@@ -67,6 +71,7 @@ const QString QSTR_GRABKB("GrabKeyboard");
 const QString QSTR_STYLEDKNOBS("StyledKnobs");
 const QString QSTR_ALWAYSONTOP("AlwaysOnTop");
 const QString QSTR_SHOWNOTENAMES("ShowNoteNames");
+const QString QSTR_SHOWSTATUSBAR("ShowStatusBar");
 const QString QSTR_RAWKEYBOARDMODE("RawKeyboardMode");
 const QString QSTR_EXTRACONTROLLERS("ExtraControllers");
 const QString QSTR_EXTRACTLPREFIX("ExtraCtl_");
@@ -75,6 +80,9 @@ const QString QSTR_VMPKINPUT("VMPK Input");
 const QString QSTR_VMPKOUTPUT("VMPK Output");
 const QString QSTR_DEFAULTINS("gmgsxg.ins");
 const QString QSTR_DRUMSCHANNEL("DrumsChannel");
+const QString QSTR_SHORTCUTS("Shortcuts");
+const QString QSTR_LANGUAGE("Language");
+const QString QSTR_VELOCITYCOLOR("VelocityColor");
 
 const QString QSTR_VMPKURL("http://vmpk.sourceforge.net");
 
@@ -87,6 +95,10 @@ const char SYSEXFILEDATA[] = "SYSEXFILEDATA\0";
 const int TOOLBARLABELMARGIN = 5;
 const int MIDIGMDRUMSCHANNEL = 9;
 const int MIDICHANNELS = 16;
+const int MIDIVELOCITY = 100;
+const int MIDIPAN = 64;
+const int MIDIVOLUME = 100;
+const int MIDIEXPRESSION = 127;
 
 #if defined(Q_WS_MAC) && defined(QT_MAC_USE_COCOA)
 #error "This program requires Qt built with the Carbon framework. Sorry."
