@@ -22,6 +22,7 @@
 #include <QtGui/QPalette>
 #include <QtGui/QGraphicsSceneMouseEvent>
 #include <QtGui/QKeyEvent>
+#include <QtCore/QDebug>
 
 #define KEYWIDTH  18
 #define KEYHEIGHT 72
@@ -41,7 +42,8 @@ PianoScene::PianoScene ( const int baseOctave,
     m_rawkbd( false ),
     m_keyPressedColor( keyPressedColor ),
     m_mousePressed( false ),
-    m_handler( NULL )
+    m_velocity( 100 ),
+    m_handler( 0 )
 {
     QBrush hilightBrush(m_keyPressedColor.isValid() ? m_keyPressedColor : QApplication::palette().highlight());
     QFont lblFont(QApplication::font());
@@ -73,12 +75,11 @@ PianoScene::PianoScene ( const int baseOctave,
             key->setPressedBrush(hilightBrush);
         m_keys.insert(i, key);
         addItem( key );
-        lbl->setHtml(noteName(i));
-        lbl->setVisible(m_showLabels);
         lbl->setFont(lblFont);
         m_labels.insert(i, lbl);
     }
     hideOrShowKeys();
+    retranslate();
 }
 
 QSize PianoScene::sizeHint() const
@@ -86,68 +87,86 @@ QSize PianoScene::sizeHint() const
     return QSize(KEYWIDTH * m_numOctaves * 7, KEYHEIGHT);
 }
 
-void PianoScene::showKeyOn( PianoKey* key )
+void PianoScene::showKeyOn( PianoKey* key, int vel )
 {
+    if (vel >= 0 && m_keyPressedColor.isValid() ) {
+        QBrush hilightBrush(m_keyPressedColor.lighter(200 - vel));
+        key->setPressedBrush(hilightBrush);
+    }
     key->setPressed(true);
 }
 
-void PianoScene::showKeyOff( PianoKey* key )
+void PianoScene::showKeyOff( PianoKey* key, int )
 {
     key->setPressed(false);
 }
 
-void PianoScene::showNoteOn( const int note )
+void PianoScene::showNoteOn( const int note, int vel )
 {
     int n = note - m_baseOctave*12 - m_transpose;
     if ((note >= m_minNote) && (note <= m_maxNote) &&
         (n >= 0) && (n < m_keys.size()))
-        showKeyOn(m_keys[n]);
+        showKeyOn(m_keys[n], vel);
 }
 
-void PianoScene::showNoteOff( const int note )
+void PianoScene::showNoteOff( const int note, int vel )
 {
     int n = note - m_baseOctave*12 - m_transpose;
     if ((note >= m_minNote) && (note <= m_maxNote) &&
         (n >= 0) && (n < m_keys.size()))
-        showKeyOff(m_keys[n]);
+        showKeyOff(m_keys[n], vel);
 }
 
-void PianoScene::keyOn( PianoKey* key )
+void PianoScene::triggerNoteOn( const int note )
 {
-    int n = m_baseOctave*12 + key->getNote() + m_transpose;
+    int n = m_baseOctave*12 + note + m_transpose;
     if ((n >= m_minNote) && (n <= m_maxNote)) {
         if (m_handler != NULL) {
             m_handler->noteOn(n);
         } else {
             emit noteOn(n);
         }
-        showKeyOn(key);
     }
 }
 
-void PianoScene::keyOff( PianoKey* key )
+void PianoScene::triggerNoteOff( const int note )
 {
-    int n = m_baseOctave*12 + key->getNote() + m_transpose;
+    int n = m_baseOctave*12 + note + m_transpose;
     if ((n >= m_minNote) && (n <= m_maxNote)) {
         if (m_handler != NULL) {
             m_handler->noteOff(n);
         } else {
             emit noteOff(n);
         }
-        showKeyOff(key);
     }
 }
 
+void PianoScene::keyOn( PianoKey* key )
+{
+    triggerNoteOn(key->getNote());
+    showKeyOn(key, m_velocity);
+}
+
+void PianoScene::keyOff( PianoKey* key )
+{
+    triggerNoteOff(key->getNote());
+    showKeyOff(key, 0);
+}
+
 void PianoScene::keyOn(const int note)
 {
     if (note >=0 && note < m_keys.size())
         keyOn(m_keys[note]);
+    else
+        triggerNoteOn(note);
 }
 
 void PianoScene::keyOff(const int note)
 {
     if (note >=0 && note < m_keys.size())
         keyOff(m_keys[note]);
+    else
+        triggerNoteOff(note);
 }
 
 PianoKey* PianoScene::getKeyForPos( const QPointF& p ) const
@@ -203,26 +222,32 @@ void PianoScene::mouseReleaseEvent ( QGraphicsSceneMouseEvent * mouseEvent )
     mouseEvent->ignore();
 }
 
-PianoKey* PianoScene::getPianoKey( const int key ) const
+int PianoScene::getNoteFromKey( const int key ) const
 {
     if (m_keybdMap != NULL) {
         KeyboardMap::ConstIterator it = m_keybdMap->constFind(key);
         if ((it != m_keybdMap->constEnd()) && (it.key() == key)) {
             int note = it.value();
-            if (note < m_keys.size())
-                return m_keys[note];
+            return note;
         }
     }
+    return -1;
+}
+
+PianoKey* PianoScene::getPianoKey( const int key ) const
+{
+    int note = getNoteFromKey(key);
+    if ((note >= 0) && (note < m_keys.size()))
+        return m_keys[note];
     return NULL;
 }
 
 void PianoScene::keyPressEvent ( QKeyEvent * keyEvent )
 {
     if ( !m_rawkbd && !keyEvent->isAutoRepeat() ) { // ignore auto-repeats
-        PianoKey* key = getPianoKey(keyEvent->key());
-        if (key != NULL) {
-            keyOn(key);
-        }
+        int note = getNoteFromKey(keyEvent->key());
+        if (note > -1)
+            keyOn(note);
     }
     keyEvent->accept();
 }
@@ -230,10 +255,9 @@ void PianoScene::keyPressEvent ( QKeyEvent * keyEvent )
 void PianoScene::keyReleaseEvent ( QKeyEvent * keyEvent )
 {
     if ( !m_rawkbd && !keyEvent->isAutoRepeat() ) { // ignore auto-repeats
-        PianoKey* key = getPianoKey(keyEvent->key());
-        if (key != NULL) {
-            keyOff(key);
-        }
+        int note = getNoteFromKey(keyEvent->key());
+        if (note > -1)
+            keyOff(note);
     }   
     keyEvent->accept();
 }
@@ -295,16 +319,12 @@ void PianoScene::setBaseOctave(const int base)
 
 QString PianoScene::noteName(const int note)
 {
-    const QString m_names_s[] = {trUtf8("C"), trUtf8("C♯"), trUtf8("D"), trUtf8("D♯"), trUtf8("E"),
-                                 trUtf8("F"), trUtf8("F♯"), trUtf8("G"), trUtf8("G♯"),
-                                 trUtf8("A"), trUtf8("A♯"), trUtf8("B")};
-    const QString m_names_f[] = {trUtf8("C"), trUtf8("D♭"), trUtf8("D"), trUtf8("E♭"), trUtf8("E"),
-                                 trUtf8("F"), trUtf8("G♭"), trUtf8("G"), trUtf8("A♭"),
-                                 trUtf8("A"), trUtf8("B♭"), trUtf8("B")};
     int num = (note + m_transpose + 12) % 12;
     int oct = m_baseOctave + ((note + m_transpose) / 12) - 1;
     if (m_noteNames.isEmpty()) {
-        QString name = m_useFlats ? m_names_f[num] : m_names_s[num];
+        QString name;
+        if (!m_names_f.isEmpty() && !m_names_s.isEmpty())
+            name = m_useFlats ? m_names_f[num] : m_names_s[num];
         return QString("%1<span style='vertical-align:sub;'>%2</span>").arg(name).arg(oct);
     } else {
         int noteIndex = note + m_transpose + 12*m_baseOctave;
@@ -370,3 +390,34 @@ void PianoScene::useStandardNoteNames()
     m_noteNames.clear();
     refreshLabels();
 }
+
+void PianoScene::retranslate()
+{
+    m_names_s.clear();
+    m_names_f.clear();
+    m_names_s << trUtf8("C")
+              << trUtf8("C♯")
+              << trUtf8("D")
+              << trUtf8("D♯")
+              << trUtf8("E")
+              << trUtf8("F")
+              << trUtf8("F♯")
+              << trUtf8("G")
+              << trUtf8("G♯")
+              << trUtf8("A")
+              << trUtf8("A♯")
+              << trUtf8("B");
+    m_names_f << trUtf8("C")
+              << trUtf8("D♭")
+              << trUtf8("D")
+              << trUtf8("E♭")
+              << trUtf8("E")
+              << trUtf8("F")
+              << trUtf8("G♭")
+              << trUtf8("G")
+              << trUtf8("A♭")
+              << trUtf8("A")
+              << trUtf8("B♭")
+              << trUtf8("B");
+    refreshLabels();
+}
