@@ -61,8 +61,8 @@ public:
     bool useFlats() const { return m_useFlats; }
     void setUseFlats(const bool use);
 
-    void showNoteOn( const int note );
-    void showNoteOff( const int note );
+    void showNoteOn( const int note, int vel = -1 );
+    void showNoteOff( const int note, int vel = -1 );
     int baseOctave() const { return m_baseOctave; }
     void setBaseOctave( const int base );
     int numOctaves() const { return m_numOctaves; }
@@ -73,20 +73,22 @@ public:
     void setRawKeyboardMode(const bool b);
     void useCustomNoteNames(const QStringList& names);
     void useStandardNoteNames();
+    int getVelocity() { return m_velocity; }
+    void setVelocity(const int velocity) { m_velocity = velocity; }
+    void retranslate();
 
 signals:
     void noteOn(int n);
     void noteOff(int n);
 
 protected:
-    void showKeyOn( PianoKey* key );
-    void showKeyOff( PianoKey* key );
+    void showKeyOn( PianoKey* key, int vel );
+    void showKeyOff( PianoKey* key, int vel );
     void keyOn( PianoKey* key );
     void keyOff( PianoKey* key );
     PianoKey* getKeyForPos( const QPointF& p ) const;
     PianoKey* getPianoKey( const int key ) const;
     QString noteName(const int note);
-
     void mouseMoveEvent ( QGraphicsSceneMouseEvent * mouseEvent );
     void mousePressEvent ( QGraphicsSceneMouseEvent * mouseEvent );
     void mouseReleaseEvent ( QGraphicsSceneMouseEvent * mouseEvent );
@@ -96,6 +98,9 @@ protected:
 private:
     void hideOrShowKeys();
     void refreshLabels();
+    void triggerNoteOn( const int note );
+    void triggerNoteOff( const int note );
+    int getNoteFromKey( const int key ) const;
     
     int m_baseOctave;
     int m_numOctaves;
@@ -107,11 +112,14 @@ private:
     bool m_rawkbd;
     QColor m_keyPressedColor;
     bool m_mousePressed;
+    int m_velocity;
     PianoHandler* m_handler;
     KeyboardMap* m_keybdMap;
     QList<PianoKey*> m_keys;
     QList<KeyLabel*> m_labels;
     QStringList m_noteNames;
+    QStringList m_names_s;
+    QStringList m_names_f;
 };
 
 #endif /*PIANOSCENE_H_*/
