@@ -30,6 +30,7 @@
 #include "midisetup.h"
 #include "kmapdialog.h"
 #include "events.h"
+#include "shortcutdialog.h"
 
 #if ENABLE_DBUS
 #include "vmpk_adaptor.h"
@@ -51,6 +52,9 @@
 #include <QtCore/QUrl>
 #include <QtCore/QString>
 #include <QtCore/QSettings>
+#include <QtCore/QTranslator>
+#include <QtCore/QLibraryInfo>
+#include <QtCore/QMapIterator>
 #include <QtCore/QDebug>
 
 VPiano::VPiano( QWidget * parent, Qt::WindowFlags flags )
@@ -62,12 +66,12 @@ VPiano::VPiano( QWidget * parent, Qt::WindowFlags flags )
     m_inputActive(false),
     m_midiThru(false),
     m_initialized(false),
-    m_dlgAbout(NULL),
-    m_dlgPreferences(NULL),
-    m_dlgMidiSetup(NULL),
-    m_dlgKeyMap(NULL),
-    m_dlgExtra(NULL),
-    m_dlgRiffImport(NULL)
+    m_dlgAbout(0),
+    m_dlgPreferences(0),
+    m_dlgMidiSetup(0),
+    m_dlgKeyMap(0),
+    m_dlgExtra(0),
+    m_dlgRiffImport(0)
 {
 #if ENABLE_DBUS
     new VmpkAdaptor(this);
@@ -75,10 +79,20 @@ VPiano::VPiano( QWidget * parent, Qt::WindowFlags flags )
     dbus.registerObject("/", this);
     dbus.registerService("net.sourceforge.vmpk");
 #endif
+    m_trq = new QTranslator(this);
+    m_trp = new QTranslator(this);
+    m_trq->load( QSTR_QTPX + configuredLanguage(),
+                 QLibraryInfo::location(QLibraryInfo::TranslationsPath) );
+    m_trp->load( QSTR_VMPKPX + configuredLanguage(),
+                 VPiano::localeDirectory() );
+    QCoreApplication::installTranslator(m_trq);
+    QCoreApplication::installTranslator(m_trp);
     ui.setupUi(this);
-    ui.actionStatusBar->setChecked(false);
+    initLanguages();
+
     connect(ui.actionAbout, SIGNAL(triggered()), SLOT(slotAbout()));
-    connect(ui.actionAbout_Qt, SIGNAL(triggered()), SLOT(slotAboutQt()));
+    connect(ui.actionAboutQt, SIGNAL(triggered()), SLOT(slotAboutQt()));
+    connect(ui.actionAboutTranslation, SIGNAL(triggered()), SLOT(slotAboutTranslation()));
     connect(ui.actionConnections, SIGNAL(triggered()), SLOT(slotConnections()));
     connect(ui.actionPreferences, SIGNAL(triggered()), SLOT(slotPreferences()));
     connect(ui.actionEditKM, SIGNAL(triggered()), SLOT(slotEditKeyboardMap()));
@@ -87,6 +101,7 @@ VPiano::VPiano( QWidget * parent, Qt::WindowFlags flags )
     connect(ui.actionImportSoundFont, SIGNAL(triggered()), SLOT(slotImportSF()));
     connect(ui.actionEditExtraControls, SIGNAL(triggered()), SLOT(slotEditExtraControls()));
     connect(ui.actionNoteNames, SIGNAL(triggered()), SLOT(slotShowNoteNames()));
+    connect(ui.actionShortcuts, SIGNAL(triggered()), SLOT(slotShortcuts()));
     ui.pianokeybd->setPianoHandler(this);
     initialization();
 }
@@ -94,11 +109,11 @@ VPiano::VPiano( QWidget * parent, Qt::WindowFlags flags )
 VPiano::~VPiano()
 {
     try {
-        if (m_midiout != NULL) {
+        if (m_midiout != 0) {
             m_midiout->closePort();
             delete m_midiout;
         }
-        if (m_midiin != NULL) {
+        if (m_midiin != 0) {
             if (m_inputActive) {
                 m_midiin->cancelCallback();
                 m_inputActive = false;
@@ -117,6 +132,7 @@ void VPiano::initialization()
     if ((m_initialized = initMidi())) {
         refreshConnections();
         readSettings();
+        createLanguageMenu();
         initToolBars();
         applyPreferences();
         applyConnections();
@@ -134,7 +150,7 @@ void midiCallback( double /*deltatime*/,
                    std::vector< unsigned char > *message,
                    void *userData )
 {
-    QEvent* ev = NULL;
+    QEvent* ev = 0;
     VPiano* instance = static_cast<VPiano*>(userData);
     instance->midiThru(message);
     unsigned char status = message->at(0) & MASK_STATUS;
@@ -147,9 +163,9 @@ void midiCallback( double /*deltatime*/,
                 unsigned char midi_note = message->at(1);
                 unsigned char vel = message->at(2);
                 if ((status == STATUS_NOTEOFF) || (vel == 0))
-                    ev = new NoteOffEvent(midi_note);
+                    ev = new NoteOffEvent(midi_note, vel);
                 else
-                    ev = new NoteOnEvent(midi_note);
+                    ev = new NoteOnEvent(midi_note, vel);
             }
             break;
         case STATUS_POLYAFT: {
@@ -181,7 +197,7 @@ void midiCallback( double /*deltatime*/,
             break;
         }
     }
-    if (ev != NULL)
+    if (ev != 0)
         QApplication::postEvent(instance, ev);
 }
 
@@ -202,18 +218,18 @@ bool VPiano::initMidi()
         int nInPorts = m_midiin->getPortCount();
         if (nInPorts == 0) {
             delete m_midiin;
-            m_midiin = NULL;
+            m_midiin = 0;
         }
 #endif
 #if defined(__LINUX_ALSASEQ__) || defined(__MACOSX_CORE__)
-        if (m_midiout != NULL)
+        if (m_midiout != 0)
             m_midiout->openVirtualPort(QSTR_VMPKOUTPUT.toStdString());
-        if (m_midiin != NULL)
+        if (m_midiin != 0)
             m_midiin->openVirtualPort(QSTR_VMPKINPUT.toStdString());
 #else //if defined(__WINDOWS_MM__) || defined(__IRIX_MD__)
         m_midiout->openPort( m_currentOut = 0 );
 #endif
-        if (m_midiin != NULL) {
+        if (m_midiin != 0) {
             // ignore SYX, clock and active sense
             m_midiin->ignoreTypes(true,true,true);
             m_midiin->setCallback( &midiCallback, this );
@@ -229,42 +245,46 @@ bool VPiano::initMidi()
 
 void VPiano::initToolBars()
 {
-    QLabel *lbl;
+    //QLabel *lbl;
     m_dialStyle = new ClassicStyle();
     m_dialStyle->setParent(this);
     // Notes tool bar
-    ui.toolBarNotes->addWidget(lbl = new QLabel(tr("Channel:"), this));
-    lbl->setMargin(TOOLBARLABELMARGIN);
+    m_lblChannel = new QLabel(tr("Channel:"), this);
+    ui.toolBarNotes->addWidget(m_lblChannel);
+    m_lblChannel->setMargin(TOOLBARLABELMARGIN);
     m_sboxChannel = new QSpinBox(this);
     m_sboxChannel->setMinimum(1);
     m_sboxChannel->setMaximum(MIDICHANNELS);
     m_sboxChannel->setValue(m_channel + 1);
     m_sboxChannel->setFocusPolicy(Qt::NoFocus);
     ui.toolBarNotes->addWidget(m_sboxChannel);
-    ui.toolBarNotes->addWidget(lbl = new QLabel(tr("Base Octave:"), this));
-    lbl->setMargin(TOOLBARLABELMARGIN);
+    m_lblBaseOctave = new QLabel(tr("Base Octave:"), this);
+    ui.toolBarNotes->addWidget(m_lblBaseOctave);
+    m_lblBaseOctave->setMargin(TOOLBARLABELMARGIN);
     m_sboxOctave = new QSpinBox(this);
     m_sboxOctave->setMinimum(0);
     m_sboxOctave->setMaximum(9);
     m_sboxOctave->setValue(m_baseOctave);
     m_sboxOctave->setFocusPolicy(Qt::NoFocus);
     ui.toolBarNotes->addWidget(m_sboxOctave);
-    ui.toolBarNotes->addWidget(lbl = new QLabel(tr("Transpose:"), this));
-    lbl->setMargin(TOOLBARLABELMARGIN);
+    m_lblTranspose = new QLabel(tr("Transpose:"), this);
+    ui.toolBarNotes->addWidget(m_lblTranspose);
+    m_lblTranspose->setMargin(TOOLBARLABELMARGIN);
     m_sboxTranspose = new QSpinBox(this);
     m_sboxTranspose->setMinimum(-11);
     m_sboxTranspose->setMaximum(11);
     m_sboxTranspose->setValue(m_transpose);
     m_sboxTranspose->setFocusPolicy(Qt::NoFocus);
     ui.toolBarNotes->addWidget(m_sboxTranspose);
-    ui.toolBarNotes->addWidget(lbl = new QLabel(tr("Velocity:"), this));
-    lbl->setMargin(TOOLBARLABELMARGIN);
+    m_lblVelocity = new QLabel(tr("Velocity:"), this);
+    ui.toolBarNotes->addWidget(m_lblVelocity);
+    m_lblVelocity->setMargin(TOOLBARLABELMARGIN);
     m_Velocity = new Knob(this);
     m_Velocity->setFixedSize(32, 32);
     m_Velocity->setStyle(dlgPreferences()->getStyledWidgets()? m_dialStyle : 0);
     m_Velocity->setMinimum(0);
     m_Velocity->setMaximum(127);
-    m_Velocity->setDefaultValue(100);
+    m_Velocity->setDefaultValue(MIDIVELOCITY);
     m_Velocity->setDialMode(Knob::LinearMode);
     m_Velocity->setValue(m_velocity);
     m_Velocity->setToolTip(QString::number(m_velocity));
@@ -278,15 +298,33 @@ void VPiano::initToolBars()
              SLOT(slotTransposeValueChanged(int)) );
     connect( m_Velocity, SIGNAL(valueChanged(int)),
              SLOT(slotVelocityValueChanged(int)) );
+    connect( ui.actionChannelUp, SIGNAL(triggered()),
+             m_sboxChannel, SLOT(stepUp()) );
+    connect( ui.actionChannelDown, SIGNAL(triggered()),
+             m_sboxChannel, SLOT(stepDown()) );
+    connect( ui.actionOctaveUp, SIGNAL(triggered()),
+             m_sboxOctave, SLOT(stepUp()) );
+    connect( ui.actionOctaveDown, SIGNAL(triggered()),
+             m_sboxOctave, SLOT(stepDown()) );
+    connect( ui.actionTransposeUp, SIGNAL(triggered()),
+             m_sboxTranspose, SLOT(stepUp()) );
+    connect( ui.actionTransposeDown, SIGNAL(triggered()),
+             m_sboxTranspose, SLOT(stepDown()) );
+    connect( ui.actionVelocityUp, SIGNAL(triggered()),
+             SLOT(slotVelocityUp()) );
+    connect( ui.actionVelocityDown, SIGNAL(triggered()),
+             SLOT(slotVelocityDown()) );
     // Controllers tool bar
-    ui.toolBarControllers->addWidget(lbl = new QLabel(tr("Control:"), this));
-    lbl->setMargin(TOOLBARLABELMARGIN);
+    m_lblControl = new QLabel(tr("Control:"), this);
+    ui.toolBarControllers->addWidget(m_lblControl);
+    m_lblControl ->setMargin(TOOLBARLABELMARGIN);
     m_comboControl = new QComboBox(this);
     m_comboControl->setSizeAdjustPolicy(QComboBox::AdjustToContents);
     m_comboControl->setFocusPolicy(Qt::NoFocus);
     ui.toolBarControllers->addWidget(m_comboControl);
-    ui.toolBarControllers->addWidget(lbl = new QLabel(tr("Value:"), this));
-    lbl->setMargin(TOOLBARLABELMARGIN);
+    m_lblValue = new QLabel(tr("Value:"), this);
+    ui.toolBarControllers->addWidget(m_lblValue);
+    m_lblValue->setMargin(TOOLBARLABELMARGIN);
     m_Control= new Knob(this);
     m_Control->setFixedSize(32, 32);
     m_Control->setStyle(dlgPreferences()->getStyledWidgets()? m_dialStyle : 0);
@@ -303,8 +341,9 @@ void VPiano::initToolBars()
     connect( m_Control, SIGNAL(sliderMoved(int)),
              SLOT(slotControlSliderMoved(int)) );
     // Pitch bender tool bar
-    ui.toolBarBender->addWidget(lbl = new QLabel(tr("Bender:"), this));
-    lbl->setMargin(TOOLBARLABELMARGIN);
+    m_lblBender = new QLabel(tr("Bender:"), this);
+    ui.toolBarBender->addWidget(m_lblBender);
+    m_lblBender->setMargin(TOOLBARLABELMARGIN);
     m_bender = new QSlider(this);
     m_bender->setOrientation(Qt::Horizontal);
     m_bender->setMaximumWidth(200);
@@ -319,14 +358,16 @@ void VPiano::initToolBars()
     connect( m_bender, SIGNAL(sliderReleased()),
              SLOT(slotBenderSliderReleased()) );
     // Programs tool bar
-    ui.toolBarPrograms->addWidget(lbl = new QLabel(tr("Bank:"), this));
-    lbl->setMargin(TOOLBARLABELMARGIN);
+    m_lblBank = new QLabel(tr("Bank:"), this);
+    ui.toolBarPrograms->addWidget(m_lblBank);
+    m_lblBank->setMargin(TOOLBARLABELMARGIN);
     m_comboBank = new QComboBox(this);
     m_comboBank->setSizeAdjustPolicy(QComboBox::AdjustToContents);
     m_comboBank->setFocusPolicy(Qt::NoFocus);
     ui.toolBarPrograms->addWidget(m_comboBank);
-    ui.toolBarPrograms->addWidget(lbl = new QLabel(tr("Program:"), this));
-    lbl->setMargin(TOOLBARLABELMARGIN);
+    m_lblProgram = new QLabel(tr("Program:"), this);
+    ui.toolBarPrograms->addWidget(m_lblProgram);
+    m_lblProgram->setMargin(TOOLBARLABELMARGIN);
     m_comboProg = new QComboBox(this);
     m_comboProg->setSizeAdjustPolicy(QComboBox::AdjustToContents);
     m_comboProg->setFocusPolicy(Qt::NoFocus);
@@ -355,6 +396,23 @@ void VPiano::initToolBars()
              SLOT(slotResetBender()));
     connect( ui.actionEditExtra, SIGNAL(triggered()),
              SLOT(slotEditExtraControls()));
+    // Tools actions
+    connect( ui.actionNextBank, SIGNAL(triggered()),
+             SLOT(slotBankNext()) );
+    connect( ui.actionPreviousBank, SIGNAL(triggered()),
+             SLOT(slotBankPrev()) );
+    connect( ui.actionNextProgram, SIGNAL(triggered()),
+             SLOT(slotProgramNext()) );
+    connect( ui.actionPreviousProgram, SIGNAL(triggered()),
+             SLOT(slotProgramPrev()) );
+    connect( ui.actionNextController, SIGNAL(triggered()),
+             SLOT(slotControllerNext()) );
+    connect( ui.actionPreviousController, SIGNAL(triggered()),
+             SLOT(slotControllerPrev()) );
+    connect( ui.actionControllerDown, SIGNAL(triggered()),
+             SLOT(slotControllerDown()) );
+    connect( ui.actionControllerUp, SIGNAL(triggered()),
+             SLOT(slotControllerUp()) );
     /* connect( ui.actionEditPrograms, SIGNAL(triggered()),
              SLOT(slotEditPrograms())); */
 }
@@ -389,12 +447,12 @@ QByteArray VPiano::readSysexDataFile(const QString& fileName)
 
 void VPiano::initExtraControllers()
 {
-    QWidget *w = NULL;
-    QCheckBox *chkbox = NULL;
-    Knob *knob = NULL;
-    QSpinBox *spin = NULL;
-    QSlider *slider = NULL;
-    QToolButton *button = NULL;
+    QWidget *w = 0;
+    QCheckBox *chkbox = 0;
+    Knob *knob = 0;
+    QSpinBox *spin = 0;
+    QSlider *slider = 0;
+    QToolButton *button = 0;
     foreach(const QString& s, m_extraControls) {
         QString lbl;
         int control = 0;
@@ -415,7 +473,7 @@ void VPiano::initExtraControllers()
         switch(type) {
         case 0:
             chkbox = new QCheckBox(this);
-            chkbox->setStyle(dlgPreferences()->getStyledWidgets() ? m_dialStyle : NULL);
+            chkbox->setStyle(dlgPreferences()->getStyledWidgets() ? m_dialStyle : 0);
             chkbox->setProperty(MIDICTLONVALUE, maxValue);
             chkbox->setProperty(MIDICTLOFFVALUE, minValue);
             chkbox->setChecked(bool(value));
@@ -425,7 +483,7 @@ void VPiano::initExtraControllers()
         case 1:
             knob = new Knob(this);
             knob->setFixedSize(32, 32);
-            knob->setStyle(dlgPreferences()->getStyledWidgets() ? m_dialStyle : NULL);
+            knob->setStyle(dlgPreferences()->getStyledWidgets() ? m_dialStyle : 0);
             knob->setMinimum(minValue);
             knob->setMaximum(maxValue);
             knob->setValue(value);
@@ -472,9 +530,9 @@ void VPiano::initExtraControllers()
             w = button;
             break;
         default:
-            w = NULL;
+            w = 0;
         }
-        if (w != NULL) {
+        if (w != 0) {
             if (!lbl.isEmpty() && type < 4) {
                 QLabel *qlbl = new QLabel(lbl, this);
                 qlbl->setMargin(TOOLBARLABELMARGIN);
@@ -499,7 +557,7 @@ void VPiano::readSettings()
 
     settings.beginGroup(QSTR_PREFERENCES);
     m_channel = settings.value(QSTR_CHANNEL, 0).toInt();
-    m_velocity = settings.value(QSTR_VELOCITY, 100).toInt();
+    m_velocity = settings.value(QSTR_VELOCITY, MIDIVELOCITY).toInt();
     m_baseOctave = settings.value(QSTR_BASEOCTAVE, 3).toInt();
     m_transpose = settings.value(QSTR_TRANSPOSE, 0).toInt();
     int num_octaves = settings.value(QSTR_NUMOCTAVES, 5).toInt();
@@ -510,6 +568,8 @@ void VPiano::readSettings()
     bool styledKnobs = settings.value(QSTR_STYLEDKNOBS, true).toBool();
     bool alwaysOnTop = settings.value(QSTR_ALWAYSONTOP, false).toBool();
     bool showNames = settings.value(QSTR_SHOWNOTENAMES, false).toBool();
+    bool showStatusBar = settings.value(QSTR_SHOWSTATUSBAR, false).toBool();
+    bool velocityColor = settings.value(QSTR_VELOCITYCOLOR, true).toBool();
     int drumsChannel = settings.value(QSTR_DRUMSCHANNEL, MIDIGMDRUMSCHANNEL).toInt();
     settings.endGroup();
 
@@ -519,7 +579,10 @@ void VPiano::readSettings()
     dlgPreferences()->setGrabKeyboard(grabKb);
     dlgPreferences()->setStyledWidgets(styledKnobs);
     dlgPreferences()->setAlwaysOnTop(alwaysOnTop);
+    dlgPreferences()->setVelocityColor(velocityColor);
     ui.actionNoteNames->setChecked(showNames);
+    ui.actionStatusBar->setChecked(showStatusBar);
+    ui.pianokeybd->setVelocity( velocityColor ? m_velocity : MIDIVELOCITY );
     slotShowNoteNames();
     if (!insFileName.isEmpty()) {
         dlgPreferences()->setInstrumentsFileName(insFileName);
@@ -538,7 +601,7 @@ void VPiano::readSettings()
     inEnabled = true;
 #endif
 
-    if (m_midiin == NULL) {
+    if (m_midiin == 0) {
         dlgMidiSetup()->inputNotAvailable();
     } else {
         dlgMidiSetup()->setInputEnabled(inEnabled);
@@ -581,6 +644,21 @@ void VPiano::readSettings()
     }
     settings.endGroup();
 
+    settings.beginGroup(QSTR_SHORTCUTS);
+    QList<QAction *> actions = findChildren<QAction *> ();
+    QListIterator<QAction *> iter(actions);
+    while (iter.hasNext()) {
+        QAction *pAction = iter.next();
+        if (pAction->objectName().isEmpty())
+            continue;
+        const QString& sKey = '/' + pAction->objectName();
+        const QString& sValue = settings.value('/' + sKey).toString();
+        if (sValue.isEmpty())
+            continue;
+        pAction->setShortcut(QKeySequence(sValue));
+    }
+    settings.endGroup();
+
     ui.pianokeybd->getKeyboardMap()->setRawMode(false);
     ui.pianokeybd->getRawKeyboardMap()->setRawMode(true);
     if (!mapFile.isEmpty() && mapFile != QSTR_DEFAULT) {
@@ -608,6 +686,7 @@ void VPiano::writeSettings()
     settings.setValue(QSTR_VELOCITY, m_velocity);
     settings.setValue(QSTR_BASEOCTAVE, m_baseOctave);
     settings.setValue(QSTR_TRANSPOSE, m_transpose);
+    settings.setValue(QSTR_LANGUAGE, m_language);
     settings.setValue(QSTR_NUMOCTAVES, dlgPreferences()->getNumOctaves());
     settings.setValue(QSTR_INSTRUMENTSDEFINITION, dlgPreferences()->getInstrumentsFileName());
     settings.setValue(QSTR_INSTRUMENTNAME, dlgPreferences()->getInstrumentName());
@@ -616,7 +695,9 @@ void VPiano::writeSettings()
     settings.setValue(QSTR_STYLEDKNOBS, dlgPreferences()->getStyledWidgets());
     settings.setValue(QSTR_ALWAYSONTOP, dlgPreferences()->getAlwaysOnTop());
     settings.setValue(QSTR_SHOWNOTENAMES, ui.actionNoteNames->isChecked());
+    settings.setValue(QSTR_SHOWSTATUSBAR, ui.actionStatusBar->isChecked());
     settings.setValue(QSTR_DRUMSCHANNEL, dlgPreferences()->getDrumsChannel());
+    settings.setValue(QSTR_VELOCITYCOLOR, dlgPreferences()->getVelocityColor());
     settings.endGroup();
 
     settings.beginGroup(QSTR_CONNECTIONS);
@@ -659,6 +740,22 @@ void VPiano::writeSettings()
     }
     settings.endGroup();
 
+    settings.beginGroup(QSTR_SHORTCUTS);
+    QList<QAction *> actions = findChildren<QAction *> ();
+    QListIterator<QAction *> iter(actions);
+    while (iter.hasNext()) {
+        QAction *pAction = iter.next();
+        if (pAction->objectName().isEmpty())
+            continue;
+        const QString& sKey = '/' + pAction->objectName();
+        const QString& sValue = QString(pAction->shortcut());
+        if (!sValue.isEmpty())
+            settings.setValue(sKey, sValue);
+        else if (settings.contains(sKey))
+                settings.remove(sKey);
+    }
+    settings.endGroup();
+
     settings.sync();
 }
 
@@ -674,7 +771,8 @@ void VPiano::customEvent ( QEvent *event )
     if ( event->type() == NoteOnEventType ) {
         NoteOnEvent *ev = static_cast<NoteOnEvent*>(event);
         int n = ev->getNote();
-        ui.pianokeybd->showNoteOn(n);
+        int v = (dlgPreferences()->getVelocityColor() ? ev->getValue() : MIDIVELOCITY );
+        ui.pianokeybd->showNoteOn(n, v);
 #ifdef ENABLE_DBUS
         emit event_noteon(n);
 #endif
@@ -699,8 +797,18 @@ void VPiano::customEvent ( QEvent *event )
         ControlChangeEvent *ev = static_cast<ControlChangeEvent*>(event);
         int ctl = ev->getController();
         int val = ev->getValue();
-        updateController(ctl, val);
-        updateExtraController(ctl, val);
+        switch (ctl) {
+        case CTL_ALL_SOUND_OFF:
+        case CTL_ALL_NOTES_OFF:
+            ui.pianokeybd->allKeysOff();
+            break;
+        case CTL_RESET_ALL_CTL:
+            initializeAllControllers();
+            break;
+        default:
+            updateController(ctl, val);
+            updateExtraController(ctl, val);
+        }
 #ifdef ENABLE_DBUS
         emit event_controlchange(ctl, val);
 #endif
@@ -827,6 +935,11 @@ void VPiano::sendController(const int controller, const int value)
 void VPiano::resetAllControllers()
 {
     sendController(CTL_RESET_ALL_CTL, 0);
+    initializeAllControllers();
+}
+
+void VPiano::initializeAllControllers()
+{
     int index = m_comboControl->currentIndex();
     int ctl = m_comboControl->itemData(index).toInt();
     int val = m_ctlState[m_channel][ctl];
@@ -877,7 +990,7 @@ void VPiano::sendProgramChange(const int program)
 
 void VPiano::sendBankChange(const int bank)
 {
-    int method = (m_ins != NULL) ? m_ins->bankSelMethod() : 0;
+    int method = (m_ins != 0) ? m_ins->bankSelMethod() : 0;
     int lsb, msb;
     switch (method) {
     case 0:
@@ -984,6 +1097,7 @@ void VPiano::slotVelocityValueChanged(int value)
 {
     m_velocity = value;
     setWidgetTip(m_Velocity, value);
+    ui.pianokeybd->setVelocity(dlgPreferences()->getVelocityColor() ? value : MIDIVELOCITY);
 }
 
 void VPiano::slotExtraController(const int value)
@@ -1041,7 +1155,7 @@ void VPiano::refreshConnections()
     try {
         dlgMidiSetup()->clearCombos();
         // inputs
-        if (m_midiin == NULL) {
+        if (m_midiin == 0) {
             dlgMidiSetup()->inputNotAvailable();
             dlgMidiSetup()->setInputEnabled(false);
         } else {
@@ -1091,7 +1205,7 @@ void VPiano::applyConnections()
             m_midiout->openPort(i);
         }
         m_currentOut = i;
-        if (m_midiin != NULL) {
+        if (m_midiin != 0) {
             nInPorts = m_midiin->getPortCount();
             i = dlgMidiSetup()->selectedInput();
             if (m_inputActive && (i != m_currentIn)) {
@@ -1116,7 +1230,7 @@ void VPiano::applyConnections()
 
 void VPiano::initControllers(int channel)
 {
-    if (m_ins != NULL) {
+    if (m_ins != 0) {
         InstrumentData controls = m_ins->control();
         InstrumentData::ConstIterator it, end;
         it = controls.constBegin();
@@ -1125,13 +1239,13 @@ void VPiano::initControllers(int channel)
             int ctl = it.key();
             switch (ctl) {
             case CTL_VOLUME:
-                m_ctlState[channel][CTL_VOLUME] = 100;
+                m_ctlState[channel][CTL_VOLUME] = MIDIVOLUME;
                 break;
             case CTL_PAN:
-                m_ctlState[channel][CTL_PAN] = 64;
+                m_ctlState[channel][CTL_PAN] = MIDIPAN;
                 break;
             case CTL_EXPRESSION:
-                m_ctlState[channel][CTL_EXPRESSION] = 127;
+                m_ctlState[channel][CTL_EXPRESSION] = MIDIEXPRESSION;
                 break;
             default:
                 m_ctlState[channel][ctl] = 0;
@@ -1144,7 +1258,7 @@ void VPiano::populateControllers()
 {
     m_comboControl->blockSignals(true);
     m_comboControl->clear();
-    if (m_ins != NULL) {
+    if (m_ins != 0) {
         InstrumentData controls = m_ins->control();
         InstrumentData::ConstIterator it, end = controls.constEnd();
         for( it = controls.constBegin(); it != end; ++it )
@@ -1165,6 +1279,7 @@ void VPiano::applyPreferences()
     }
     ui.pianokeybd->setKeyPressedColor(dlgPreferences()->getKeyPressedColor());
     ui.pianokeybd->setRawKeyboardMode(dlgPreferences()->getRawKeyboard());
+    ui.pianokeybd->setVelocity(dlgPreferences()->getVelocityColor() ? m_velocity : MIDIVELOCITY );
 
     KeyboardMap* map = dlgPreferences()->getKeyboardMap();
     if (!map->getFileName().isEmpty() && map->getFileName() != QSTR_DEFAULT )
@@ -1197,7 +1312,7 @@ void VPiano::applyPreferences()
 
 void VPiano::populateInstruments()
 {
-    m_ins = NULL;
+    m_ins = 0;
     m_comboBank->clear();
     m_comboProg->clear();
     if (!dlgPreferences()->getInstrumentsFileName().isEmpty() &&
@@ -1206,7 +1321,7 @@ void VPiano::populateInstruments()
             m_ins = dlgPreferences()->getDrumsInstrument();
         else
             m_ins = dlgPreferences()->getInstrument();
-        if (m_ins != NULL) {
+        if (m_ins != 0) {
             //qDebug() << "Instrument Name:" << m_ins->instrumentName();
             //qDebug() << "Bank Selection method: " << m_ins->bankSelMethod();
             InstrumentPatches patches = m_ins->patches();
@@ -1270,6 +1385,15 @@ QString VPiano::dataDirectory()
     return QString();
 }
 
+QString VPiano::localeDirectory()
+{
+#ifdef Q_OS_LINUX
+    return VPiano::dataDirectory() + "locale/";
+#else
+    return VPiano::dataDirectory();
+#endif
+}
+
 void VPiano::slotEditKeyboardMap()
 {
     KeyboardMap* map;
@@ -1294,7 +1418,7 @@ void VPiano::populatePrograms(int bank)
     if (bank < 0)
         return;
     m_comboProg->clear();
-    if (m_ins != NULL) {
+    if (m_ins != 0) {
         InstrumentData patch = m_ins->patch(bank);
         InstrumentData::ConstIterator k;
         for( k = patch.constBegin(); k != patch.constEnd(); ++k )
@@ -1351,7 +1475,7 @@ void VPiano::slotTransposeValueChanged(const int transpose)
 
 void VPiano::updateNoteNames(bool drums)
 {
-    if (drums && (m_ins != NULL)) {
+    if (drums && (m_ins != 0)) {
         int b = m_lastBank[m_channel];
         int p = m_lastProg[m_channel];
         const InstrumentData& notes = m_ins->notes(b, p);
@@ -1400,7 +1524,7 @@ void VPiano::updateController(int ctl, int val)
         m_Control->setToolTip(QString::number(val));
     }
     m_ctlState[m_channel][ctl] = val;
-    if ((ctl == CTL_MSB || ctl == CTL_LSB ) && m_ins != NULL) {
+    if ((ctl == CTL_MSB || ctl == CTL_LSB ) && m_ins != 0) {
         if (m_ins->bankSelMethod() == 0)
             m_lastBank[m_channel] = m_ctlState[m_channel][CTL_MSB] << 7 |
                                     m_ctlState[m_channel][CTL_LSB];
@@ -1497,7 +1621,7 @@ void VPiano::releaseKb()
 void VPiano::slotHelpContents()
 {
     QStringList hlps;
-    QLocale loc = QLocale::system();
+    QLocale loc(configuredLanguage());
     QStringList lc = loc.name().split("_");
     hlps += QString("help_%1.html").arg(loc.name());
     if (lc.count() > 1)
@@ -1524,11 +1648,11 @@ void VPiano::updateStyles()
 {
     QList<Knob *> allKnobs = findChildren<Knob *> ();
     foreach(Knob* knob, allKnobs) {
-        knob->setStyle(dlgPreferences()->getStyledWidgets() ? m_dialStyle : NULL);
+        knob->setStyle(dlgPreferences()->getStyledWidgets() ? m_dialStyle : 0);
     }
     QList<QCheckBox *> allChkbox = ui.toolBarExtra->findChildren<QCheckBox *> ();
     foreach(QCheckBox* chkbox, allChkbox) {
-        chkbox->setStyle(dlgPreferences()->getStyledWidgets() ? m_dialStyle : NULL);
+        chkbox->setStyle(dlgPreferences()->getStyledWidgets() ? m_dialStyle : 0);
     }
 }
 
@@ -1559,7 +1683,7 @@ void VPiano::slotEditExtraControls()
 
 About* VPiano::dlgAbout()
 {
-    if (m_dlgAbout == NULL) {
+    if (m_dlgAbout == 0) {
         m_dlgAbout = new About(this);
     }
     return m_dlgAbout;
@@ -1567,7 +1691,7 @@ About* VPiano::dlgAbout()
 
 Preferences* VPiano::dlgPreferences()
 {
-    if (m_dlgPreferences == NULL) {
+    if (m_dlgPreferences == 0) {
         m_dlgPreferences = new Preferences(this);
     }
     return m_dlgPreferences;
@@ -1575,7 +1699,7 @@ Preferences* VPiano::dlgPreferences()
 
 MidiSetup* VPiano::dlgMidiSetup()
 {
-    if (m_dlgMidiSetup == NULL) {
+    if (m_dlgMidiSetup == 0) {
         m_dlgMidiSetup = new MidiSetup(this);
     }
     return m_dlgMidiSetup;
@@ -1583,7 +1707,7 @@ MidiSetup* VPiano::dlgMidiSetup()
 
 KMapDialog* VPiano::dlgKeyMap()
 {
-    if (m_dlgKeyMap == NULL) {
+    if (m_dlgKeyMap == 0) {
         m_dlgKeyMap = new KMapDialog(this);
     }
     return m_dlgKeyMap;
@@ -1591,7 +1715,7 @@ KMapDialog* VPiano::dlgKeyMap()
 
 DialogExtraControls* VPiano::dlgExtra()
 {
-    if (m_dlgExtra == NULL) {
+    if (m_dlgExtra == 0) {
         m_dlgExtra = new DialogExtraControls(this);
     }
     return m_dlgExtra;
@@ -1599,7 +1723,7 @@ DialogExtraControls* VPiano::dlgExtra()
 
 RiffImportDlg* VPiano::dlgRiffImport()
 {
-    if (m_dlgRiffImport == NULL) {
+    if (m_dlgRiffImport == 0) {
         m_dlgRiffImport = new RiffImportDlg(this);
     }
     return m_dlgRiffImport;
@@ -1609,7 +1733,7 @@ void VPiano::setWidgetTip(QWidget* w, int val)
 {
     QString tip = QString::number(val);
     w->setToolTip(tip);
-    QToolTip::showText(QCursor::pos(), tip, this);
+    QToolTip::showText(w->parentWidget()->mapToGlobal(w->pos()), tip);
 }
 
 void VPiano::slotShowNoteNames()
@@ -1686,14 +1810,14 @@ void VPiano::connect_thru(bool value)
 void VPiano::noteoff(int note)
 {
     sendNoteOff(note);
-    NoteOffEvent *ev = new NoteOffEvent(note);
+    NoteOffEvent *ev = new NoteOffEvent(note, 0);
     QApplication::postEvent(this, ev);
 }
 
 void VPiano::noteon(int note)
 {
     sendNoteOn(note);
-    NoteOnEvent *ev = new NoteOnEvent(note);
+    NoteOnEvent *ev = new NoteOnEvent(note, m_velocity);
     QApplication::postEvent(this, ev);
 }
 
@@ -1742,3 +1866,219 @@ void VPiano::pitchwheel(int value)
 }
 
 #endif /* ENABLE_DBUS */
+
+void VPiano::slotShortcuts()
+{
+    ShortcutDialog shcutDlg(findChildren<QAction*>());
+    releaseKb();
+    shcutDlg.exec();
+    grabKb();
+}
+
+void VPiano::slotBankNext()
+{
+    int index = m_comboBank->currentIndex();
+    if (index < m_comboBank->count()-1) {
+        m_comboBank->setCurrentIndex(++index);
+        slotComboBankActivated(index);
+    }
+}
+
+void VPiano::slotBankPrev()
+{
+    int index = m_comboBank->currentIndex();
+    if (index > 0) {
+        m_comboBank->setCurrentIndex(--index);
+        slotComboBankActivated(index);
+    }
+}
+
+void VPiano::slotProgramNext()
+{
+    int index = m_comboProg->currentIndex();
+    if (index < m_comboProg->count()-1) {
+        m_comboProg->setCurrentIndex(++index);
+        slotComboProgActivated(index);
+    }
+}
+
+void VPiano::slotProgramPrev()
+{
+    int index = m_comboProg->currentIndex();
+    if (index > 0) {
+        m_comboProg->setCurrentIndex(--index);
+        slotComboProgActivated(index);
+    }
+}
+
+void VPiano::slotControllerNext()
+{
+    int index = m_comboControl->currentIndex();
+    if (index < m_comboControl->count()-1)
+        m_comboControl->setCurrentIndex(++index);
+}
+
+void VPiano::slotControllerPrev()
+{
+    int index = m_comboControl->currentIndex();
+    if (index > 0)
+        m_comboControl->setCurrentIndex(--index);
+}
+
+void VPiano::slotVelocityUp()
+{
+    m_Velocity->triggerAction(Knob::SliderPageStepAdd);
+}
+
+void VPiano::slotVelocityDown()
+{
+    m_Velocity->triggerAction(Knob::SliderPageStepSub);
+}
+
+void VPiano::slotControllerUp()
+{
+    m_Control->triggerAction(Knob::SliderPageStepAdd);
+    slotControlSliderMoved(m_Control->value());
+}
+
+void VPiano::slotControllerDown()
+{
+    m_Control->triggerAction(Knob::SliderPageStepSub);
+    slotControlSliderMoved(m_Control->value());
+}
+
+QString VPiano::configuredLanguage()
+{
+    if (m_language.isEmpty()) {
+        QSettings settings;
+        QString defLang = QLocale::system().name();
+        settings.beginGroup(QSTR_PREFERENCES);
+        m_language = settings.value(QSTR_LANGUAGE, defLang).toString();
+        settings.endGroup();
+        //qDebug() << Q_FUNC_INFO << m_language;
+    }
+    return m_language;
+}
+
+void VPiano::slotSwitchLanguage(QAction *action)
+{
+    QString lang = action->data().toString();
+    if ( QMessageBox::question (this, tr("Language Changed"),
+            tr("The language for this application is going to change to %1. "
+               "Do you want to continue?").arg(m_supportedLangs[lang]),
+            QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes )
+    {
+        m_language = lang;
+        retranslateUi();
+    } else {
+        m_currentLang->setChecked(true);
+    }
+}
+
+void VPiano::createLanguageMenu()
+{
+    QString currentLang = configuredLanguage();
+    QActionGroup *languageGroup = new QActionGroup(this);
+    connect(languageGroup, SIGNAL(triggered(QAction *)),
+            SLOT(slotSwitchLanguage(QAction *)));
+    QDir dir(VPiano::localeDirectory());
+    QStringList fileNames = dir.entryList(QStringList(QSTR_VMPKPX + "*.qm"));
+    QStringList locales;
+    locales << "en";
+    foreach (const QString& fileName, fileNames) {
+        QString locale = fileName;
+        locale.remove(0, locale.indexOf('_') + 1);
+        locale.truncate(locale.lastIndexOf('.'));
+        locales << locale;
+    }
+    locales.sort();
+    foreach (const QString& loc, locales) {
+        QAction *action = new QAction(m_supportedLangs.value(loc), this);
+        action->setCheckable(true);
+        action->setData(loc);
+        ui.menuLanguage->addAction(action);
+        languageGroup->addAction(action);
+        if (currentLang.startsWith(loc)) {
+            action->setChecked(true);
+            m_currentLang = action;
+        }
+    }
+}
+
+void VPiano::slotAboutTranslation()
+{
+    QString common = tr("<p>VMPK is developed and translated thanks to the "
+        "volunteer work of many people from around the world. If you want to "
+        "join the team or have any question, please visit the forums at "
+        "<a href='http://sourceforge.net/projects/vmpk/forums'>SourceForge</a>"
+        "</p>");
+    QString currentLang = configuredLanguage();
+    bool supported(false);
+    if (!currentLang.startsWith("en")) {
+        QMapIterator<QString,QString> it(m_supportedLangs);
+        while (it.hasNext())  {
+            it.next();
+            if (currentLang.startsWith(it.key())) {
+                supported = true;
+                break;
+            }
+        }
+    }
+    if (supported)
+        QMessageBox::information(this, tr("Translation"),
+            tr("<p>Translation by TRANSLATOR_NAME_AND_EMAIL</p>%1").arg(common));
+    else
+        QMessageBox::information(this, tr("Translation Information"), common);
+}
+
+void VPiano::retranslateUi()
+{
+    m_trq->load( QSTR_QTPX + configuredLanguage(),
+                 QLibraryInfo::location(QLibraryInfo::TranslationsPath) );
+    m_trp->load( QSTR_VMPKPX + configuredLanguage(),
+                 VPiano::localeDirectory() );
+    ui.retranslateUi(this);
+    ui.pianokeybd->retranslate();
+    initLanguages();
+    ui.menuLanguage->clear();
+    createLanguageMenu();
+    retranslateToolbars();
+    if (m_dlgAbout != 0)
+        m_dlgAbout->retranslateUi();
+    if (m_dlgPreferences != 0)
+        m_dlgPreferences->retranslateUi();
+    if (m_dlgMidiSetup != 0)
+        m_dlgMidiSetup->retranslateUi();
+    if (m_dlgKeyMap != 0)
+        m_dlgKeyMap->retranslateUi();
+    if (m_dlgExtra != 0)
+        m_dlgExtra->retranslateUi();
+    if (m_dlgRiffImport != 0)
+        m_dlgRiffImport->retranslateUi();
+}
+
+void VPiano::initLanguages()
+{
+    m_supportedLangs.clear();
+    m_supportedLangs.insert(QLatin1String("cs"), tr("Czech"));
+    m_supportedLangs.insert(QLatin1String("de"), tr("German"));
+    m_supportedLangs.insert(QLatin1String("en"), tr("English"));
+    m_supportedLangs.insert(QLatin1String("es"), tr("Spanish"));
+    m_supportedLangs.insert(QLatin1String("fr"), tr("French"));
+    m_supportedLangs.insert(QLatin1String("ru"), tr("Russian"));
+    m_supportedLangs.insert(QLatin1String("tr"), tr("Turkish"));
+    m_supportedLangs.insert(QLatin1String("zh_CN"), tr("Chinese"));
+}
+
+void VPiano::retranslateToolbars()
+{
+    m_lblBank->setText(tr("Bank:"));
+    m_lblBaseOctave->setText(tr("Base Octave:"));
+    m_lblBender->setText(tr("Bender:"));
+    m_lblChannel->setText(tr("Channel:"));
+    m_lblControl->setText(tr("Control:"));
+    m_lblProgram->setText(tr("Program:"));
+    m_lblTranspose->setText(tr("Transpose:"));
+    m_lblValue->setText(tr("Value:"));
+    m_lblVelocity->setText(tr("Velocity:"));
+}
