@@ -23,6 +23,8 @@
 #include "pianoscene.h"
 #include <QtGui/QMainWindow>
 
+class QTranslator;
+class QLabel;
 class QComboBox;
 class QSpinBox;
 class QSlider;
@@ -48,6 +50,7 @@ public:
     int getInputChannel();
     void midiThru(std::vector<unsigned char> *message) const;
     bool isInitialized() const { return m_initialized; }
+    void retranslateUi();
 
     // PianoHandler methods
     void noteOn(const int midiNote);
@@ -55,6 +58,7 @@ public:
 
     // static methods
     static QString dataDirectory();
+    static QString localeDirectory();
 
 #if ENABLE_DBUS
 
@@ -93,6 +97,7 @@ Q_SIGNALS:
 protected Q_SLOTS:
     void slotAbout();
     void slotAboutQt();
+    void slotAboutTranslation();
     void slotConnections();
     void slotPreferences();
     void slotEditKeyboardMap();
@@ -116,6 +121,18 @@ protected Q_SLOTS:
     void slotVelocityValueChanged(int value);
     void slotChannelValueChanged(const int channel);
     void slotComboControlCurrentIndexChanged(const int index);
+    void slotShortcuts();
+    void slotVelocityUp();
+    void slotVelocityDown();
+    void slotBankNext();
+    void slotBankPrev();
+    void slotProgramNext();
+    void slotProgramPrev();
+    void slotControllerNext();
+    void slotControllerPrev();
+    void slotControllerUp();
+    void slotControllerDown();
+    void slotSwitchLanguage(QAction *action);
     //void slotEditPrograms();
     //void slotDebugDestroyed(QObject *obj);
 
@@ -142,6 +159,7 @@ private:
     void initExtraControllers();
     void initControllers(int channel);
     void resetAllControllers();
+    void initializeAllControllers();
     void allNotesOff();
     void sendController(const int controller, const int value);
     void sendBankChange(const int bank);
@@ -163,6 +181,8 @@ private:
     void updateNoteNames(bool drums);
     void setWidgetTip(QWidget* w, int val);
     QByteArray readSysexDataFile(const QString& fileName);
+    void createLanguageMenu();
+    QString configuredLanguage();
 
     About *dlgAbout();
     Preferences *dlgPreferences();
@@ -171,6 +191,9 @@ private:
     DialogExtraControls *dlgExtra();
     RiffImportDlg *dlgRiffImport();
 
+    void initLanguages();
+    void retranslateToolbars();
+
     RtMidiOut* m_midiout;
     RtMidiIn* m_midiin;
     int m_currentOut;
@@ -188,6 +211,15 @@ private:
 
     Ui::VPiano ui;
 
+    QLabel* m_lblBank;
+    QLabel* m_lblBaseOctave;
+    QLabel* m_lblBender;
+    QLabel* m_lblChannel;
+    QLabel* m_lblControl;
+    QLabel* m_lblProgram;
+    QLabel* m_lblTranspose;
+    QLabel* m_lblValue;
+    QLabel* m_lblVelocity;
     QSpinBox* m_sboxChannel;
     QSpinBox* m_sboxOctave;
     QSpinBox* m_sboxTranspose;
@@ -208,6 +240,10 @@ private:
     int m_velocity;
     int m_baseOctave;
     int m_transpose;
+    QString m_language;
+    QMap<QString, QString> m_supportedLangs;
+    QTranslator *m_trq, *m_trp;
+    QAction *m_currentLang;
 };
 
 #endif // VPIANO_H
